import java.io.*;
import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.util.Properties;
import java.util.StringTokenizer;


abstract class LatticeModelEA_application  extends EA_application
{

	/*
   IvpEA_application constructor:
	 */
	String lscape_name;
	LatticeModelEA_application(Properties props)
	{
		super(props);
	}



	/*
  run: runs the given EA on the given landscape
	 */

	void run()
	{	
		/************** Core run command ************************************************/
		ThreadMXBean tb = ManagementFactory.getThreadMXBean();
  		long starttime = tb.getCurrentThreadCpuTime();
		for (int i=1; i<= run_count;i++){
			ea.run(simulation_limit);
			ea.outputStats();
		}
		if (run_count==1) ea.print_best();
		long stoptime = tb.getCurrentThreadCpuTime();
		long timeused = (stoptime - starttime) / 1000000;	// to get milliseconds
		System.out.println("Number of Function evaluations = "+EC4.maxNFE);
		System.out.println("Time used(ms) = "+timeused);
		/************** END - Core run command *******************************************/
		
		double EArray[] 						= Util.arrayListOfDoubleToArray(ea.maxEnergyList);
		double ValidEArray[] 					= Util.arrayListOfDoubleToArray(ea.maxValidEnergyList);
		double distanceNumOfIndivArray[] 		= Util.arrayListOfDoubleToArray(ea.distanceNumOfIndivList);
		double mirrorDistanceNumOfIndivArray[]	= Util.arrayListOfDoubleToArray(ea.mirrorDistanceNumOfIndivList);
		double distanceDiversity[]				= Util.arrayListOfDoubleToArray(ea.distanceDiversityList);
		double mirrorDistanceDiversity[]		= Util.arrayListOfDoubleToArray(ea.mirrorDistanceDiversityList);
		double avgContactSeperationMeanArray[]  = Util.arrayListOfDoubleToArray(ea.avgContactSeperationMeanList);
		double avgContactSeperationSDArray[]    = Util.arrayListOfDoubleToArray(ea.avgContactSeperationSDList);
		double volumeArray[]					= Util.arrayListOfDoubleToArray(ea.volumeList);
		double volumeSDArray[]					= Util.arrayListOfDoubleToArray(ea.volumeSDList);
		
		System.out.println("E max    ="+Util.max(EArray));
		System.out.println("E mean   ="+Util.mean(EArray));
		System.out.println("E SD     ="+Util.SDKnuth(EArray));
		System.out.println();
		System.out.println("ValidE max  ="+Util.max(ValidEArray));
		System.out.println("ValidE mean ="+Util.mean(ValidEArray));
		System.out.println("ValidE SD   ="+Util.SDKnuth(ValidEArray));
		System.out.println();
		System.out.println("Distance Number of Indiv mean ="+Util.mean(distanceNumOfIndivArray));
		System.out.println("Distance Number of Indiv SD   ="+Util.SDKnuth(distanceNumOfIndivArray));
		System.out.println("MDistance Number of Indiv mean="+Util.mean(mirrorDistanceNumOfIndivArray));
		System.out.println("MDistance Number of Indiv SD  ="+Util.SDKnuth(mirrorDistanceNumOfIndivArray));
		
		System.out.println();
		System.out.println("Distance Diversity mean ="+Util.mean(distanceDiversity));
		System.out.println("Distance Diversity SD   ="+Util.SDKnuth(distanceDiversity));
		System.out.println("MDistance Diversity mean="+Util.mean(mirrorDistanceDiversity));
		System.out.println("MDistance Diversity SD  ="+Util.SDKnuth(mirrorDistanceDiversity));
		System.out.println();
		System.out.println("AvgContactSeperationMean mean="+Util.mean(avgContactSeperationMeanArray));
		System.out.println("AvgContactSeperationMean SD  ="+Util.SDKnuth(avgContactSeperationMeanArray));
		System.out.println("AvgContactSeperationSD   mean="+Util.mean(avgContactSeperationSDArray));
		System.out.println("AvgContactSeperationSD   SD  ="+Util.SDKnuth(avgContactSeperationSDArray));
		System.out.println("Volume mean   ="+Util.mean(volumeArray));
		System.out.println("Volume SD     ="+Util.SDKnuth(volumeArray));
		System.out.println("VolumeSD mean ="+Util.mean(volumeSDArray));
		System.out.println("VolumeSD SD   ="+Util.SDKnuth(volumeSDArray));
		
		try{
			  // Create file 
			  FileWriter fstream = new FileWriter("log/LatticeModelLog.csv", true);
			  BufferedWriter out = new BufferedWriter(fstream);
			  out.write(Util.now()+","+ea.representation_name+"-"+ea.getClass().getName()+","+lscape_name+","+EC4.maxNFE+","+timeused+",,");
			  out.write(Util.max(EArray)+","+Util.mean(EArray)+","+Util.SDKnuth(EArray)+",,");
			  out.write(Util.max(ValidEArray)+","+Util.mean(ValidEArray)+","+Util.SDKnuth(ValidEArray)+",,");
			  out.write(Util.mean(distanceNumOfIndivArray)+","+Util.SDKnuth(distanceNumOfIndivArray)+",,");
			  out.write(Util.mean(mirrorDistanceNumOfIndivArray)+","+Util.SDKnuth(mirrorDistanceNumOfIndivArray)+",,");
			  
			  out.write("\n");
			  out.flush();
			  //Close the output stream
			  out.close();
			  
			  FileWriter testFstream = new FileWriter("log/LatticeModelTestLog.csv", true);
			  BufferedWriter testOut = new BufferedWriter(testFstream);
			  testOut.write(Util.now()+","+ea.getClass().getName()+","+lscape_name+","+EC4.maxNFE+","+timeused+",,");
			  testOut.write(Util.mean(distanceDiversity)+","+Util.SDKnuth(distanceDiversity)+",,");
			  testOut.write(Util.mean(mirrorDistanceDiversity)+","+Util.SDKnuth(mirrorDistanceDiversity)+",,");
			  testOut.write(Util.mean(avgContactSeperationMeanArray)+","+Util.SDKnuth(avgContactSeperationMeanArray)+",,");
			  testOut.write(Util.mean(avgContactSeperationSDArray)+","+Util.SDKnuth(avgContactSeperationSDArray)+",,");
			  testOut.write(Util.mean(volumeArray)+","+Util.SDKnuth(volumeArray)+",,");
			  testOut.write(Util.mean(volumeSDArray)+","+Util.SDKnuth(volumeSDArray)+",,");
			  testOut.write("\n");
			  testOut.flush();
			  testOut.close();
		  }catch (Exception e){//Catch exception if any
			  System.err.println("Error: " + e.getMessage());
		  }
	}

}
